@@ -1,8 +1,12 @@
 package App::Dispatcher;
+use strict;
+use warnings;
 use Carp qw/croak confess/;
 use Getopt::Long::Descriptive qw/describe_options/;
 use File::Spec::Functions qw/catdir catfile splitdir/;
 use File::Find;
+use File::Basename;
+use Storable qw/dclone/;
 use Template;
 use Perl::Tidy qw/perltidy/;
 use Pod::Tidy;
@@ -11,8 +15,7 @@ use File::ShareDir qw/dist_dir/;
 use Sub::Exporter -setup => {
     exports => [
         qw/
-          app_dispatcher
-          app_dispatcher_opts
+          run_app_dispatcher
           /,
     ],
     groups => {
@@ -24,20 +27,18 @@ use Sub::Exporter -setup => {
 
 };
 use lib 'lib';
-use strict;
-use warnings;
 no warnings 'redefine';    # due to bootstrap/build time effects
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 # partially stolen from ExtUtils::MakeMaker
 sub _abstract {
-    my ( $class, $pm_file ) = @_;
+    my ( $class, $file ) = @_;
 
-    confess '_abstract($class,$pm_file)' unless ( $class and $pm_file );
+    confess '_abstract($class,$file)' unless ( $class and $file );
 
     my $result;
-    open my $fh, "<", $pm_file or return "(unknown)";
+    open my $fh, "<", $file or return;
 
     local $/ = "\n";
     my $inpod;
@@ -52,14 +53,21 @@ sub _abstract {
         $result = $1;
         last;
     }
-    return $result || "(unknown)";
+    return $result;
 }
 
-sub app_dispatcher {
-    my $class = shift || croak 'app_dispatcher($class,$opt)';
-    my $opt   = shift || croak 'app_dispatcher($class,$opt)';
+# Helper sub.
+sub run_app_dispatcher {
+    my $class = shift;
+    local @ARGV = ( @_, qw/--class/, $class );
+    my ( $opt, $usage ) = describe_options( 'x', opt_spec(), arg_spec() );
 
-    $opt->{add_mtime_check} = 1 if $opt->{add_debug};
+    _app_dispatcher( $class, $opt );
+}
+
+sub _app_dispatcher {
+    my $class = shift || croak '_app_dispatcher($class,$opt)';
+    my $opt   = shift || croak '_app_dispatcher($class,$opt)';
 
     my $outdir = catdir( 'lib', split( '::', $class ) );
     my $output = catfile( $outdir, 'Dispatcher.pm' );
@@ -84,6 +92,7 @@ sub app_dispatcher {
     if ( !$classfile ) {
         die "Cannot find class '$class' (searched: @paths)\n";
     }
+    require $classfile;
 
     my @plugins = ($class);
     my @files   = ($classfile);
@@ -112,16 +121,31 @@ sub app_dispatcher {
     }
 
     my $ref = {
-        class           => '__App__Dispatcher__Test__Class__',
-        version         => $VERSION,
-        add_debug       => $opt->add_debug,
-        add_mtime_check => $opt->add_mtime_check,
-        structure       => {},
+        class     => '__App__Dispatcher__Test__Class__',
+        version   => $VERSION,
+        add_debug => $opt->add_debug,
+        structure => {},
     };
 
     # Make sure @INC is good for 'require'
     unshift @INC, @{ $opt->inc }, @paths;
 
+    my @gopts;
+
+    if ( $opt->add_short_help ) {
+        push( @gopts, [ 'help|h', 'print usage message and exit' ] );
+    }
+    elsif ( $opt->add_help ) {
+        push( @gopts, [ 'help', 'print usage message and exit' ] );
+    }
+
+    if ( $opt->add_debug ) {
+        push( @gopts,
+            [ 'debug-dispatcher', 'print App::Dispatcher debug information' ] );
+    }
+
+    push( @gopts, eval { $class->gopt_spec } );
+
     foreach my $plugin (@plugins) {
         my $file = shift @files;
         if ( $opt->verbose ) {
@@ -135,25 +159,19 @@ sub app_dispatcher {
         $usage =~ s/::/ /g;
 
         my @opt;
-        if ( $opt->add_help ) {
-            push( @opt, [ 'help|h', 'print usage message and exit' ] );
-        }
-        if ( $opt->add_debug and $plugin eq $class ) {
-            push(
-                @opt,
-                [
-                    'debug-dispatcher',
-                    'print App::Dispatcher debug information'
-                ]
-            );
+
+        my $abstract = _abstract( $plugin, $file );
+        if ( !$abstract ) {
+            ( my $pod = $file ) =~ s/\.pm/\.pod/;
+            $abstract = _abstract( $plugin, $pod ) || '(unknown)',;
         }
 
         my $refp = $ref->{structure}->{$plugin} = {
-            name     => $name,
-            class    => $plugin,
-            abstract => _abstract( $plugin, $file ),
-            order         => 2**31 - 1,    # just a big number, nothing special
-            opt_spec      => \@opt,
+            name          => $name,
+            class         => $plugin,
+            abstract      => $abstract,
+            order         => 2**31 - 1,     # just a big number, nothing special
+            opt_spec      => dclone \@gopts,
             arg_spec      => [],
             require_order => 0,
             getopt_conf   => [],
@@ -190,8 +208,8 @@ sub app_dispatcher {
             $refp->{usage_desc} .= ' ' . join(
                 ' ',
                 map {
-                    ( my $x = $_->[0] ) =~ s/(.*)[|=].*/$1/;
-                    exists $_->[2]->{required} ? "<$x>" : "[<$x>]";
+                    ( my $x = uc $_->[0] ) =~ s/(.*)[|=].*/$1/;
+                    exists $_->[2]->{required} ? "$x" : "[$x]";
                   } @{ $refp->{arg_spec} }
             );
         }
@@ -220,17 +238,27 @@ sub app_dispatcher {
 
     print "Generating $output\n";
 
+    my @tpaths;
+    foreach my $path (
+        'share',
+        dirname($0) . '/../share',
+        eval { dist_dir('App-Dispatcher') }
+      )
+    {
+        next unless -d $path;
+        push( @tpaths, $path );
+    }
+
     my $template = Template->new(
-        INCLUDE_PATH =>
-          [ 'share', '../share', eval { dist_dir('App-Dispatcher') } || () ],
-        EVAL_PERL => 1,
+        INCLUDE_PATH => \@tpaths,
+        EVAL_PERL    => 1,
     ) || die Template->error;
 
     my $txt;
     $template->process( 'Dispatcher.pm.tt', $ref, \$txt )
       || die "Template: " . $template->error . "\n";
 
-    eval "$txt";
+    eval "$txt";    ## no critic
     if ( my $err = $@ ) {
         my $i = 0;
         map { printf STDERR "%-03s $_\n", $i++ } split( /\n/, $txt );
@@ -266,9 +294,9 @@ sub app_dispatcher {
 
 sub opt_spec {
     (
-        [ 'add-help|H',        "add a '--help' option to every command", ],
-        [ 'add-debug|D',       "add a global '--debug-dispatcher' option", ],
-        [ 'add-mtime-check|M', "check for out of date command files" ],
+        [ 'add-help|H',     "add a '--help' option to every command", ],
+        [ 'add-short-help', "add a '-h' option to every command", ],
+        [ 'add-debug|D',    "add a global '--debug-dispatcher' option", ],
         [ 'inc|I=s@', "add to the perl \@INC array", { default => [] } ],
         [
             'path|p=s@',
@@ -286,181 +314,10 @@ sub arg_spec {
     );
 }
 
-sub app_dispatcher_opts {
-    local @ARGV = @_;
-    my ( $opt, $usage ) = eval {
-        describe_options(
-            '(not a user
-    command)', ( opt_spec, arg_spec )
-        );
-    };
-
-    die "Invalid options: $@" if ($@);
-    return $opt;
-}
-
 sub run {
     my ( $self, $opt ) = @_;
 
-    app_dispatcher( $opt->class, $opt );
+    _app_dispatcher( $opt->class, $opt );
 }
 
 1;
-__END__
-
-=head1 NAME
-
-App::Dispatcher - generate command-line dispatcher classes
-
-=head1 SYNOPSIS
-
-    use App::Dispatcher qw/app_dispatcher/;
-    app_dispatcher( $class, $opt );
-
-=head1 DESCRIPTION
-
-This is the interface documentation for the B<App::Dispatcher> class,
-which is generally not used directly by application authors.  To make
-use of B<App::Dispatcher> in your application you most likely want to
-be reading L<App::Dispatcher::Tutorial> instead.
-
-B<App::Dispatcher> is a the implementation for the L<app-dispatcher>(1)
-command. The implementation is contained in a single subroutine with
-that takes two mandatory arguments:
-
-=over 4
-
-=item app_dispatcher( $class, $opt )
-
-=over 4
-
-=item $class
-
-The name space under which the command classes will be searched for.
-
-=item $opt
-
-A L<Getopt::Long::Descriptive::Opts> (or equivalent) object with the
-following methods:
-
-=over 4
-
-=item add_help
-
-Add a '--help' option to every command.
-
-=item add_debug
-
-Add a global '--debug-dispatcher' option. Useful for debugging the
-actual options your command classes are receiving. Automatically turns
-on 'add_mtime_check'.
-
-=item add_mtime_check
-
-Warn at dispatch time if the command class file has a more recent
-modification date than the dispatch class.
-
-=item inc
-
-Arrayref to add to @INC.
-
-=item path
-
-Arrayref of directories to search for command classes.
-
-=item verbose
-
-Print out extra details during execution.
-
-=item dry_run
-
-Don't write out any files to disk.
-
-=item force
-
-Force the overwriting of existing writeable files.
-
-=back
-
-=back
-
-=back
-
-A successful run of app_dispatcher() results in the creation of 
-'lib/$class/Dispatcher.pm', which is based on the Command Classes found
-under 'lib/$class'.
-
-=over 4
-
-=item app_dispatcher_opts( @args )
-
-=back
-
-The command classes may have the following methods defined:
-
-=over 4
-
-=item require_order
-
-Optional. An boolean to set the L<Getopt::Long> option 'require_order'
-when true, or set option 'permute' (the default) when false (or not
-defined).
-
-=item order
-
-Optional. An integer to force the order in which commands are
-displayed.
-
-=item usage_desc
-
-Optional. The string used for error messgaes. Will be auto-generated
-from the opt_spec and arg_spec methods if not implemented.
-
-=item opt_spec
-
-Optional, but at least one of opt_spec or arg_spec should in most cases
-be implemented. A list of option definitions which will be passed to
-the describe_options method of Getopt::Long::Descriptive.
-
-=item arg_spec
-
-Optional, but at least one of opt_spec or arg_spec should in most cases
-be implemented. A list of argument definitions which will be passed to
-the describe_options method of Getopt::Long::Descriptive.
-
-=item run
-
-Optional, but it only makes sense to be left unimplemented if the
-command requires subcommands. This is the method where the real work is
-performed.
-
-=item abstract
-
-Optional. A brief description of the command. Will be pulled from the
-POD documentation if not implemented.
-
-=back
-
-B<App::Dispatcher> is also its own Command Class. That is, it
-implements the opt_spec(), arg_spec() and run() methods.
-
-=head1 SEE ALSO
-
-L<app-dispatcher>(1), L<App::Dispatcher::Tutorial>(3p)
-
-=head1 AUTHOR
-
-Mark Lawrence E<lt>nomad@null.netE<gt>
-
-=head1 COPYRIGHT AND LICENSE
-
-Copyright (C) 2011 Mark Lawrence <nomad@null.net>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-=cut
-
-z vim: set tabstop=4 expandtab:
