@@ -2,7 +2,7 @@ package Your::Command;
 
 sub require_order { 1 }
 
-sub opt_spec {
+sub gopt_spec {
     ( [ "dry-run|n", "print out SQL instead of running it" ], );
 }
 
